package cn.com.tenlion.bigdata.service.bigdatatemplatearea;

import cn.com.tenlion.bigdata.pojo.dtos.bigdatatemplatearea.BigdataTemplateAreaDTO;
import cn.com.tenlion.bigdata.pojo.vos.bigdatatemplatearea.BigdataTemplateAreaPiointVO;
import cn.com.tenlion.bigdata.pojo.vos.bigdatatemplatearea.BigdataTemplateAreaVO;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IBigdataTemplateAreaService
 * @Description: 大数据展示模板的区域设置
 * @Author: WenG
 * @Date: 2020-10-20 15:08
 * @Version: 1.0
 **/
public interface IBigdataTemplateAreaService {

    /**
     * 新增大数据展示模板的区域设置
     *
     * @param bigdataTemplateAreaVO
     * @return
     * @throws Exception
     */
    SuccessResult saveBigdataTemplateArea(BigdataTemplateAreaVO bigdataTemplateAreaVO) throws Exception;

    /**
     * 新增大数据展示模板的区域设置(APP)
     *
     * @param token
     * @param bigdataTemplateAreaVO
     * @return
     * @throws Exception
     */
    SuccessResult saveBigdataTemplateAreaByToken(String token, BigdataTemplateAreaVO bigdataTemplateAreaVO) throws Exception;

    /**
     * 新增大数据展示模板的区域设置
     *
     * @param bigdataTemplateAreaVO
     * @return bigdataTemplateAreaId
     * @throws Exception
     */
    String saveBigdataTemplateAreaReturnId(BigdataTemplateAreaVO bigdataTemplateAreaVO) throws Exception;

    /**
     * 新增大数据展示模板的区域设置(APP)
     *
     * @param token
     * @param bigdataTemplateAreaVO
     * @return bigdataTemplateAreaId
     * @throws Exception
     */
    String saveBigdataTemplateAreaByTokenReturnId(String token, BigdataTemplateAreaVO bigdataTemplateAreaVO) throws Exception;

    /**
     * 删除大数据展示模板的区域设置
     *
     * @param ids
     * @return
     * @throws RemoveException
     */
    SuccessResult removeBigdataTemplateArea(String ids) throws RemoveException;

    /**
     * 删除大数据展示模板的区域设置（物理删除）
     *
     * @param ids
     * @throws RemoveException
     */
    void deleteBigdataTemplateArea(String ids) throws RemoveException;

    /**
     * 删除大数据展示模板的区域设置(APP)
     *
     * @param token
     * @param ids
     * @return
     * @throws RemoveException
     */
    SuccessResult removeBigdataTemplateAreaByToken(String token, String ids) throws RemoveException;

    /**
     * 修改大数据展示模板的区域设置
     *
     * @param bigdataTemplateAreaId
     * @param bigdataTemplateAreaVO
     * @return
     * @throws Exception
     */
    SuccessResult updateBigdataTemplateArea(String bigdataTemplateAreaId, BigdataTemplateAreaVO bigdataTemplateAreaVO) throws Exception;

    /**
     * 修改大数据展示模板的区域设置(APP)
     *
     * @param token
     * @param bigdataTemplateAreaId
     * @param bigdataTemplateAreaVO
     * @return
     * @throws Exception
     */
    SuccessResult updateBigdataTemplateAreaByToken(String token, String bigdataTemplateAreaId, BigdataTemplateAreaVO bigdataTemplateAreaVO) throws Exception;

    /**
     * 大数据展示模板的区域设置详情(通过ID)
     *
     * @param bigdataTemplateAreaId
     * @return
     * @throws SearchException
     */
    BigdataTemplateAreaDTO getBigdataTemplateAreaById(String bigdataTemplateAreaId) throws SearchException;

    /**
     * 大数据展示模板的区域设置列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<BigdataTemplateAreaDTO> listBigdataTemplateArea(Map<String, Object> params) throws SearchException;

    /**
     * 大数据展示模板的区域设置分页列表
     *
     * @param page
     * @return
     * @throws SearchException
     */
    SuccessResultList<List<BigdataTemplateAreaDTO>> listPageBigdataTemplateArea(ListPage page) throws SearchException;

    /**
     * 大数据展示模板的区域设置统计
     *
     * @param params
     * @return
     * @throws SearchException
     */
    Integer countNumberBigdataTemplateArea(Map<String, Object> params) throws SearchException;

    /**
     * 大数据展示模板的区域设置统计
     *
     * @param params
     * @return
     * @throws SearchException
     */
    SuccessResultData<Integer> countBigdataTemplateArea(Map<String, Object> params) throws SearchException;

    List<BigdataTemplateAreaDTO> getBigdataTemplateAreaByBigdataTemplateId(String bigdataTemplateId) throws SearchException;

    SuccessResult updateBigdataTemplateAreaPoint(String bigdataTemplateAreaId, BigdataTemplateAreaPiointVO bigdataTemplateAreaPiointVO) throws Exception;

    SuccessResult copyBigdataTemplateAreaPoint(String bigdataTemplateAreaId) throws Exception;

    SuccessResult updateBigdataTemplateAreaPointModuleData(String bigdataTemplateAreaId, String templateAreaModuleData) throws Exception;
}